#This file is part of Tryton.  The COPYRIGHT file at the top level of
#this repository contains the full copyright notices and license terms.
from trytond.model import ModelView, ModelSQL, fields
from decimal import Decimal


class Template(ModelSQL, ModelView):
    _name = 'product.template'

    list_prices = fields.One2Many('product.template.list_price', 'template',
            'List Prices', states={
                'readonly': "active == False",
            })

    def _compute_composite_cost_price(self, cursor, user, template, context=None):
        cost_price = super(Template, self)._compute_composite_cost_price(
            cursor, user, template, context=context)
        for composite in template.composites:
            if composite.product.type == 'service' and composite.profit_center:
                cost_price -= Decimal(str(composite.quantity)) * \
                    composite.product.cost_price
        return cost_price

Template()


class Composite(ModelSQL, ModelView):

    _name = 'product.composite'

    profit_center = fields.Many2One(
        'sale_import.profit_center', 'Profit Center',
        depends=['service_product'], states={'invisible': "not service_product"})
    service_product = fields.Function(
        'get_service_product', type='boolean', string='Service Product',
        on_change_with=['product'])

    def get_service_product(self, cursor, user, ids, name, arg, context=None):
        res = {}
        for composite in self.browse(cursor, user, ids, context=context):
            res[composite.id] = composite.product.type == 'service'
        return res

    def on_change_with_service_product(self, cursor, user, ids, vals, context=None):
        product_obj = self.pool.get('product.product')
        if vals.get('product'):
            product = product_obj.browse(
                cursor, user, vals['product'], context=context)
            return product.type == "service"
        return False

Composite()


class ListPrice(ModelSQL, ModelView):
    'Product Template List Price'
    _name = 'product.template.list_price'
    _description = __doc__
    _rec_name = 'list_price'

    template = fields.Many2One('product.template', 'Product', required=True)
    profit_center = fields.Many2One('sale_import.profit_center',
            'Profit Center', required=True)
    list_price = fields.Property(type='numeric', string='List Price',
            digits=(16, 4))
    cost_price = fields.Function('get_cost_price', type='numeric',
            string='Cost Price', digits=(16, 4))
    margin = fields.Function('get_margin', type='numeric', string='Margin',
            digits=(16, 4))

    def __init__(self):
        super(ListPrice, self).__init__()
        self._sql_constraints = [
            ('template_profit_center_uniq', 'UNIQUE(template, profit_center)',
                'The profit center must be unique per product!'),
        ]

    def get_cost_price(self, cursor, user, ids, name, arg, context=None):
        res = {}
        for list_price in self.browse(cursor, user, ids, context=context):
            if not list_price.template.composite:
                res[list_price.id] = list_price.template.cost_price
            else:
                res[list_price.id] = Decimal('0.0')
                for composite in list_price.template.composites:
                    if composite.product.type != 'service' or \
                            composite.profit_center.id == \
                            list_price.profit_center.id:
                        res[list_price.id] += \
                                Decimal(str(composite.quantity)) * \
                                composite.product.cost_price
        return res

    def get_margin(self, cursor, user, ids, name, arg, context=None):
        res = {}
        for list_price in self.browse(cursor, user, ids, context=context):
            res[list_price.id] = list_price.list_price - \
                    list_price.cost_price
        return res

ListPrice()
